from typing import Any

from ._base import HtmlSERP, SerpItem


class BingSerp(HtmlSERP):
    provider = "bing"
    item_xpath = '//li[contains(concat(" ", normalize-space(@class), " "), " b_algo ")]'

    def parse_node(self, node: Any) -> SerpItem | None:
        title = self.first_text(node, ".//h2[1]")
        url = self.first_attr(node, ".//h2[1]/a/@href")
        author = self.first_text(node, './/div[contains(@class,"tptt")][1]')
        excerpt = self.first_text(node, './/div[contains(@class,"b_caption")]//p[1]')
        return self.make_item(title=title, excerpt=excerpt, url=url, author=author)



from typing import Any

from ._base import HtmlSERP, SerpItem


class SogouSerp(HtmlSERP):
    provider = "sogou"
    item_xpath = '//div[@id="main"]//div[contains(@class,"vrwrap") and .//h3//a[@href]]'

    def parse_node(self, node: Any) -> SerpItem | None:
        title = self.first_text(node, ".//h3[1]")
        url = self.first_attr(node, ".//h3//a[1]/@href")
        author = self.first_text(node, ".//cite[1]")
        cover_url = self.first_attr(node, ".//img[1]/@src")
        excerpt = self.first_text(
            node,
            './/*[contains(@class,"str-text-info")][1]',
            ".//p[1]",
        ) or self.fallback_excerpt(node, title)
        return self.make_item(
            title=title,
            excerpt=excerpt,
            url=url,
            author=author,
            cover_url=cover_url,
        )



from ._base import BaseSERP, SerpItem, extract_json_from_text, take_text


class ThreadsSerp(BaseSERP):
    provider = "threads"

    def pick_entry(self, entries: list[dict[str, object]]) -> dict[str, object]:
        return entries[-1]

    def parse_response(self, body: str) -> list[SerpItem]:
        payload = extract_json_from_text(body)
        root = payload["data"].get("searchResults", {})
        items: list[SerpItem] = []
        for edge in root.get("edges", []):
            thread = edge["node"]["thread"]
            post = thread["thread_items"][0]["post"]
            user = post.get("user", {})
            title = take_text(post.get("text_post_app_info", {}).get("text_fragments"))
            built = self.make_item(
                title=title,
                excerpt=title,
                url=post.get("canonical_url"),
                author=user.get("username") or user.get("full_name"),
                cover_url=user.get("profile_pic_url"),
            )
            if built is not None:
                items.append(built)
        return items



from ._base import BaseSERP, SerpItem, extract_json_from_text, strip_html


class ZhihuSerp(BaseSERP):
    provider = "zhihu"

    def parse_response(self, body: str) -> list[SerpItem]:
        payload = extract_json_from_text(body)
        items: list[SerpItem] = []
        for result in payload.get("data", []):
            if result.get("type") != "search_result":
                continue
            obj = result["object"]
            thumbnails = obj.get("thumbnail_info", {}).get("thumbnails") or []
            built = self.make_item(
                title=obj.get("title") or strip_html(obj.get("content")),
                excerpt=strip_html(obj.get("excerpt")) or strip_html(obj.get("content")),
                url=obj.get("url"),
                author=obj.get("author", {}).get("name"),
                cover_url=thumbnails[0].get("url") if thumbnails else None,
            )
            if built is not None:
                items.append(built)
        return items



from typing import Any

from ._base import HtmlSERP, SerpItem


class YandexSerp(HtmlSERP):
    provider = "yandex"
    item_xpath = '//li[contains(@class,"serp-item")]'

    def parse_node(self, node: Any) -> SerpItem | None:
        title = self.first_text(
            node,
            './/*[contains(@class,"OrganicTitleContentSpan")][1]',
            ".//h2[1]",
        )
        url = self.first_attr(node, './/a[contains(@class,"OrganicTitle-Link")][1]/@href')
        excerpt = self.first_text(node, './/*[contains(@class,"OrganicTextContentSpan")][1]')
        author = self.first_text(node, './/*[contains(@class,"OrganicHost-TitleText")][1]')
        return self.make_item(title=title, excerpt=excerpt, url=url, author=author)



import json
import re

from ._base import BaseSERP, SerpItem, clean_text, strip_html


class DuckDuckGoSerp(BaseSERP):
    provider = "duckduckgo"

    def parse_response(self, body: str) -> list[SerpItem]:
        match = re.search(r"DDG\.duckbar\.add\((\{.*?\}),null,\"index\"\)", body)
        if not match:
            return []
        payload = json.loads(match.group(1))
        abstract = clean_text(payload["data"].get("AbstractText"))
        items: list[SerpItem] = []
        for result in payload["data"].get("Results", []):
            item = self.make_item(
                title=strip_html(result.get("Text")),
                excerpt=abstract,
                url=result.get("FirstURL"),
                author=payload["data"].get("AbstractSource"),
                cover_url=result.get("Icon", {}).get("URL"),
            )
            if item is not None:
                items.append(item)
        return items



from ._base import BaseSERP, EngineConfig, HarRequestTemplate, HtmlSERP, SerpItem
from .baidu import BaiduSerp
from .bilibili import BilibiliSerp
from .bing import BingSerp
from .brave import BraveSerp
from .douyin import DouyinSerp
from .duckduckgo import DuckDuckGoSerp
from .google import GoogleSerp
from .reddit import RedditSerp
from .so import SoSerp
from .sogou import SogouSerp
from .threads import ThreadsSerp
from .tiktok import TiktokSerp
from .toutiao import ToutiaoSerp
from .weibo import WeiboSerp
from .yandex import YandexSerp
from .youtube import YouTubeSerp
from .zhihu import ZhihuSerp

__all__ = [
    "BaseSERP",
    "EngineConfig",
    "HarRequestTemplate",
    "HtmlSERP",
    "SerpItem",
    "BaiduSerp",
    "BilibiliSerp",
    "BingSerp",
    "BraveSerp",
    "DouyinSerp",
    "DuckDuckGoSerp",
    "GoogleSerp",
    "RedditSerp",
    "SoSerp",
    "SogouSerp",
    "ThreadsSerp",
    "TiktokSerp",
    "ToutiaoSerp",
    "WeiboSerp",
    "YandexSerp",
    "YouTubeSerp",
    "ZhihuSerp",
]



from typing import Any

from ._base import HtmlSERP, SerpItem


class BraveSerp(HtmlSERP):
    provider = "brave"
    item_xpath = '//div[@id="results"]//div[@data-type="web"]'

    def parse_node(self, node: Any) -> SerpItem | None:
        title = self.first_text(node, './/div[contains(@class,"title")][1]')
        url = self.first_attr(node, ".//a[@href][1]/@href")
        author = self.first_text(node, './/div[contains(@class,"site-name-content")]//div[1]')
        cover_url = self.first_attr(node, './/img[contains(@class,"favicon")][1]/@src')
        excerpt = self.first_text(
            node,
            './/div[contains(@class,"generic-snippet")]//*[contains(@class,"content")][1]',
            './/div[contains(@class,"description")][1]',
        ) or self.fallback_excerpt(node, title)
        return self.make_item(
            title=title,
            excerpt=excerpt,
            url=url,
            author=author,
            cover_url=cover_url,
        )



from seekit import get_provider
from seekit.providers._base import HarRequestTemplate


def test_google_request_template_replaces_keyword() -> None:
    template = get_provider("google").get_request_template("Latest OpenAI")

    assert isinstance(template, HarRequestTemplate)
    assert template.method == "GET"
    assert "Latest%20OpenAI" in template.url or "Latest+OpenAI" in template.url
    assert all(not name.startswith(":") for name, _ in template.headers)


def test_threads_request_template_replaces_keyword_in_body() -> None:
    template = get_provider("threads").get_request_template("fresh keyword")

    assert template.method == "POST"
    assert template.body is not None
    assert "fresh keyword" in template.body or "fresh+keyword" in template.body



from ._base import BaseSERP, SerpItem, extract_json_from_text, take_text


class YouTubeSerp(BaseSERP):
    provider = "youtube"

    def parse_response(self, body: str) -> list[SerpItem]:
        marker = "var ytInitialData = "
        start = body.find(marker)
        if start == -1:
            return []
        payload = extract_json_from_text(body[start + len(marker) :])
        sections = payload["contents"]["twoColumnSearchResultsRenderer"]["primaryContents"][
            "sectionListRenderer"
        ]["contents"]
        items: list[SerpItem] = []
        for section in sections:
            renderer = section.get("itemSectionRenderer")
            if not renderer:
                continue
            for entry in renderer["contents"]:
                video = entry.get("videoRenderer")
                if not video:
                    continue
                thumbnails = video.get("thumbnail", {}).get("thumbnails") or []
                item = self.make_item(
                    title=take_text(video.get("title")),
                    excerpt=take_text(video.get("detailedMetadataSnippets")),
                    url="https://www.youtube.com/watch?v=" + video["videoId"],
                    author=take_text(video.get("ownerText"))
                    or take_text(video.get("longBylineText")),
                    cover_url=thumbnails[-1].get("url") if thumbnails else None,
                )
                if item is not None:
                    items.append(item)
        return items



from ._base import BaseSERP, SerpItem, extract_json_from_text, strip_html


class BilibiliSerp(BaseSERP):
    provider = "bilibili"

    def parse_response(self, body: str) -> list[SerpItem]:
        payload = extract_json_from_text(body)
        items: list[SerpItem] = []
        for section in payload["data"]["result"]:
            for entry in section.get("data", []):
                if section["result_type"] == "video":
                    url = entry.get("arcurl")
                    title = strip_html(entry.get("title"))
                    excerpt = entry.get("description")
                    author = entry.get("author")
                    cover_url = entry.get("pic")
                elif section["result_type"] == "bili_user":
                    url = f"https://space.bilibili.com/{entry['mid']}"
                    title = entry.get("uname")
                    excerpt = entry.get("usign")
                    author = entry.get("uname")
                    cover_url = entry.get("upic")
                else:
                    continue
                item = self.make_item(
                    title=title,
                    excerpt=excerpt,
                    url=url,
                    author=author,
                    cover_url=cover_url,
                )
                if item is not None:
                    items.append(item)
        return items



from __future__ import annotations

from abc import ABC, abstractmethod
import base64
from collections import OrderedDict
from dataclasses import dataclass
from functools import cache
import json
from json import JSONDecoder
from pathlib import Path
import re
from typing import Any
from urllib.parse import quote, quote_plus, urljoin

import curl_cffi
import lxml.html
from pydantic import BaseModel
import yaml


@dataclass(frozen=True)
class EngineExample:
    keyword: str
    page: str


@dataclass(frozen=True)
class EngineConfig:
    name: str
    example: EngineExample
    type: str | None = None


@dataclass(frozen=True)
class HarRequestTemplate:
    method: str
    url: str
    headers: tuple[tuple[str, str], ...]
    cookies: tuple[tuple[str, str], ...]
    body: str | None = None


class SerpItem(BaseModel):
    provider: str
    title: str | None = None
    excerpt: str | None = None
    url: str | None = None
    author: str | None = None
    cover_url: str | None = None


def clean_text(value: str | None) -> str | None:
    if value is None:
        return None
    text = re.sub(r"\s+", " ", value).strip()
    return text or None


def strip_html(value: str | None) -> str | None:
    if not value:
        return None
    fragment = lxml.html.fromstring(f"<div>{value}</div>")
    return clean_text(fragment.text_content())


def take_text(value: Any) -> str | None:
    if value is None:
        return None
    if isinstance(value, str):
        return clean_text(value)
    if isinstance(value, dict):
        if "simpleText" in value:
            return clean_text(value["simpleText"])
        if "text" in value and isinstance(value["text"], str):
            return clean_text(value["text"])
        if "runs" in value:
            return clean_text("".join(run.get("text", "") for run in value["runs"]))
        if "fragments" in value:
            return clean_text(
                "".join(fragment.get("plaintext", "") for fragment in value["fragments"])
            )
    if isinstance(value, list):
        return clean_text(" ".join(part for part in (take_text(item) for item in value) if part))
    return None


def absolutize_url(value: str | None, base_url: str | None = None) -> str | None:
    value = clean_text(value)
    if not value:
        return None
    if value.startswith("//"):
        return "https:" + value
    if base_url:
        return urljoin(base_url, value)
    return value


@cache
def load_engine_configs(path: str | Path = "data/info.yaml") -> tuple[EngineConfig, ...]:
    with Path(path).open() as handle:
        payload = yaml.safe_load(handle)
    configs = []
    for entry in payload["engines"]:
        example = EngineExample(**entry["example"])
        configs.append(EngineConfig(name=entry["name"], example=example, type=entry.get("type")))
    return tuple(configs)


@cache
def load_engine_config_map(path: str | Path = "data/info.yaml") -> dict[str, EngineConfig]:
    return {config.name: config for config in load_engine_configs(path)}


def get_engine_config(name: str, path: str | Path = "data/info.yaml") -> EngineConfig:
    return load_engine_config_map(path)[name]


def load_har_entries(path: str | Path) -> list[dict[str, Any]]:
    with Path(path).open() as handle:
        payload = json.load(handle)
    return payload["log"]["entries"]


def decode_har_content(content: dict[str, Any]) -> str:
    text = content.get("text", "")
    if content.get("encoding") == "base64":
        text = base64.b64decode(text).decode("utf-8", errors="ignore")
    return text


def extract_json_from_text(text: str) -> Any:
    candidates = [index for index in (text.find("{"), text.find("[")) if index != -1]
    if not candidates:
        raise ValueError("No JSON payload found in response body.")
    start = min(candidates)
    payload, _ = JSONDecoder().raw_decode(text[start:])
    return payload


def build_har_request_template(entry: dict[str, Any]) -> HarRequestTemplate:
    request = entry["request"]
    headers: list[tuple[str, str]] = []
    for header in request.get("headers", []):
        name = header["name"]
        if name.startswith(":"):
            continue
        headers.append((name, header["value"]))
    cookies = tuple((cookie["name"], cookie["value"]) for cookie in request.get("cookies", []))
    post_data = request.get("postData")
    body = post_data.get("text") if post_data else None
    return HarRequestTemplate(
        method=request["method"],
        url=request["url"],
        headers=tuple(headers),
        cookies=cookies,
        body=body,
    )


def replace_keyword(value: str, source_keyword: str, target_keyword: str) -> str:
    replacements = (
        (source_keyword, target_keyword),
        (quote(source_keyword), quote(target_keyword)),
        (quote(source_keyword, safe=""), quote(target_keyword, safe="")),
        (quote_plus(source_keyword), quote_plus(target_keyword)),
    )
    result = value
    for source, target in replacements:
        result = result.replace(source, target)
    return result


class BaseSERP(ABC):
    provider: str = ""
    base_url: str | None = None

    def get_engine_config(self) -> EngineConfig:
        return get_engine_config(self.provider)

    def get_har_path(self) -> Path:
        config = self.get_engine_config()
        return Path("data/pages") / config.example.page

    def pick_entry(self, entries: list[dict[str, Any]]) -> dict[str, Any]:
        return entries[0]

    def parse_har_file(self, path: str | Path) -> list[SerpItem]:
        entry = self.pick_entry(load_har_entries(path))
        body = decode_har_content(entry["response"]["content"])
        return self.parse_response(body)

    def get_request_template(self, keyword: str) -> HarRequestTemplate:
        config = self.get_engine_config()
        entry = self.pick_entry(load_har_entries(self.get_har_path()))
        template = build_har_request_template(entry)
        return self.apply_keyword_to_template(
            template,
            source_keyword=config.example.keyword,
            target_keyword=keyword,
        )

    def apply_keyword_to_template(
        self,
        template: HarRequestTemplate,
        *,
        source_keyword: str,
        target_keyword: str,
    ) -> HarRequestTemplate:
        return HarRequestTemplate(
            method=template.method,
            url=replace_keyword(template.url, source_keyword, target_keyword),
            headers=template.headers,
            cookies=template.cookies,
            body=(
                replace_keyword(template.body, source_keyword, target_keyword)
                if template.body is not None
                else None
            ),
        )

    def request(self, keyword: str) -> str:
        template = self.get_request_template(keyword)
        response = curl_cffi.requests.request(
            method=template.method,
            url=template.url,
            headers=OrderedDict(template.headers),
            cookies=OrderedDict(template.cookies),
            data=template.body,
            impersonate="chrome120",
        )
        return response.text

    @abstractmethod
    def parse_response(self, body: str) -> list[SerpItem]:
        raise NotImplementedError

    def query(self, keyword: str) -> list[SerpItem]:
        return self.parse_response(self.request(keyword))

    def make_item(
        self,
        *,
        title: str | None,
        excerpt: str | None,
        url: str | None,
        author: str | None = None,
        cover_url: str | None = None,
        base_url: str | None = None,
    ) -> SerpItem | None:
        title = clean_text(title)
        excerpt = clean_text(excerpt)
        author = clean_text(author)
        url = absolutize_url(url, base_url or self.base_url)
        cover_url = absolutize_url(cover_url, base_url or self.base_url)
        if not any([title, excerpt, url]):
            return None
        return SerpItem(
            provider=self.provider,
            title=title,
            excerpt=excerpt,
            url=url,
            author=author,
            cover_url=cover_url,
        )


class HtmlSERP(BaseSERP):
    item_xpath: str = ""

    def parse_response(self, body: str) -> list[SerpItem]:
        tree = lxml.html.fromstring(body)
        items: list[SerpItem] = []
        for node in tree.xpath(self.item_xpath):
            item = self.parse_node(node)
            if item is not None:
                items.append(item)
        return items

    @abstractmethod
    def parse_node(self, node: Any) -> SerpItem | None:
        raise NotImplementedError

    def first_text(self, node: Any, *xpaths: str) -> str | None:
        for xpath in xpaths:
            values = node.xpath(xpath)
            if not values:
                continue
            first = values[0]
            if isinstance(first, str):
                text = clean_text(first)
            else:
                text = clean_text(first.text_content())
            if text:
                return text
        return None

    def first_attr(self, node: Any, xpath: str) -> str | None:
        values = node.xpath(xpath)
        if not values:
            return None
        return clean_text(values[0])

    def fallback_excerpt(self, node: Any, title: str | None) -> str | None:
        text = clean_text(node.text_content())
        if not text:
            return None
        if title:
            text = clean_text(text.replace(title, "", 1))
        return text



from ._base import BaseSERP, SerpItem, extract_json_from_text, strip_html


class WeiboSerp(BaseSERP):
    provider = "weibo"

    def parse_response(self, body: str) -> list[SerpItem]:
        payload = extract_json_from_text(body)
        items: list[SerpItem] = []
        for card in payload["data"].get("cards", []):
            if card.get("card_type") != 9:
                continue
            post = card["mblog"]
            user = post.get("user", {})
            built = self.make_item(
                title=strip_html(post.get("text")),
                excerpt=strip_html(post.get("text")),
                url=f"https://m.weibo.cn/detail/{post['id']}",
                author=user.get("screen_name"),
                cover_url=user.get("profile_image_url"),
            )
            if built is not None:
                items.append(built)
        return items



from __future__ import annotations

import argparse
from collections.abc import Sequence
import sys
import textwrap

from . import PROVIDERS, SerpItem, search


def build_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="seekit",
        description="Search a provider and print normalized results in a table.",
    )
    parser.add_argument("provider", choices=sorted(PROVIDERS))
    parser.add_argument("keyword", nargs="+", help="Search keyword")
    parser.add_argument(
        "--limit",
        type=int,
        default=10,
        help="Maximum number of rows to print (default: 10)",
    )
    return parser


def truncate(value: str | None, width: int) -> str:
    text = (value or "").strip()
    if len(text) <= width:
        return text
    if width <= 3:
        return text[:width]
    return text[: width - 3] + "..."


def format_table(items: Sequence[SerpItem]) -> str:
    headers = ("#", "Provider", "Title", "Author", "URL")
    rows = []
    for index, item in enumerate(items, start=1):
        rows.append(
            (
                str(index),
                item.provider,
                truncate(item.title, 42),
                truncate(item.author, 18),
                truncate(item.url, 60),
            )
        )

    widths = [len(header) for header in headers]
    for row in rows:
        for index, value in enumerate(row):
            widths[index] = max(widths[index], len(value))

    divider = "-+-".join("-" * width for width in widths)
    lines = [
        " | ".join(header.ljust(widths[index]) for index, header in enumerate(headers)),
        divider,
    ]
    for row in rows:
        lines.append(" | ".join(value.ljust(widths[index]) for index, value in enumerate(row)))
    return "\n".join(lines)


def format_detail(items: Sequence[SerpItem]) -> str:
    blocks: list[str] = []
    for index, item in enumerate(items, start=1):
        lines = [
            f"[{index}] {item.title or '(no title)'}",
            f"provider: {item.provider}",
        ]
        if item.author:
            lines.append(f"author: {item.author}")
        if item.url:
            lines.append(f"url: {item.url}")
        if item.excerpt:
            lines.append("excerpt:")
            lines.extend(textwrap.wrap(item.excerpt, width=100))
        blocks.append("\n".join(lines))
    return "\n\n".join(blocks)


def main(argv: Sequence[str] | None = None) -> int:
    parser = build_parser()
    args = parser.parse_args(argv)
    keyword = " ".join(args.keyword)

    try:
        results = search(keyword, provider=args.provider)
    except Exception as exc:
        print(f"seekit: {exc}", file=sys.stderr)
        return 1

    if args.limit < 1:
        print("seekit: --limit must be at least 1", file=sys.stderr)
        return 2

    items = results[: args.limit]
    if not items:
        print("No results.")
        return 0

    print(format_table(items))
    print()
    print(format_detail(items))
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



from typing import Any

from ._base import HtmlSERP, SerpItem


class BaiduSerp(HtmlSERP):
    provider = "baidu"
    item_xpath = (
        '//div[@id="content_left"]//div['
        '(contains(@class,"result") or contains(@class,"result-op")) and .//h3//a[@href]'
        "]"
    )

    def parse_node(self, node: Any) -> SerpItem | None:
        title = self.first_text(node, ".//h3[1]")
        url = self.first_attr(node, ".//h3//a[1]/@href")
        author = self.first_text(
            node,
            './/*[contains(@class,"cos-tag")][1]',
            './/*[contains(@class,"c-color-gray2")][1]',
        )
        excerpt = self.first_text(
            node,
            './/*[contains(@class,"c-span-last")][1]',
            ".//p[1]",
            ".//div[.//text()][2]",
        ) or self.fallback_excerpt(node, title)
        return self.make_item(title=title, excerpt=excerpt, url=url, author=author)



from typing import Any

from ._base import HtmlSERP, SerpItem


class SoSerp(HtmlSERP):
    provider = "so"
    item_xpath = '//li[contains(@class,"res-list")]'

    def parse_node(self, node: Any) -> SerpItem | None:
        title = self.first_text(node, ".//h3[1]")
        url = self.first_attr(node, ".//h3//a[1]/@href")
        author = self.first_text(node, ".//cite[1]")
        excerpt = self.first_text(node, './/p[contains(@class,"res-desc")][1]', ".//p[1]")
        return self.make_item(title=title, excerpt=excerpt, url=url, author=author)



from typing import Any

from ._base import HtmlSERP, SerpItem


class RedditSerp(HtmlSERP):
    provider = "reddit"
    base_url = "https://www.reddit.com"
    item_xpath = '//a[@data-testid="post-title-text"]'

    def parse_node(self, node: Any) -> SerpItem | None:
        title = self.first_text(node, ".")
        url = self.first_attr(node, "./@href")
        excerpt = self.first_text(
            node.getparent(),
            './/a[contains(@class,"line-clamp-2")][1]',
        )
        author = self.first_text(
            node.getparent(),
            './/a[starts-with(@href,"/r/")][1]',
        )
        return self.make_item(
            title=title,
            excerpt=excerpt,
            url=url,
            author=author,
            base_url=self.base_url,
        )



import re

from ._base import BaseSERP, SerpItem


class ToutiaoSerp(BaseSERP):
    provider = "toutiao"

    def parse_response(self, body: str) -> list[SerpItem]:
        pattern = re.compile(
            r'"title":"(?P<title>[^"]+?)".{0,1200}?"abstract":"(?P<excerpt>[^"]+?)".{0,1200}?"open_url":"(?P<url>http[^"]+)".{0,1200}?"media_name":"(?P<author>[^"]*?)"',
            re.S,
        )
        items: list[SerpItem] = []
        for match in pattern.finditer(body):
            built = self.make_item(
                title=match.group("title"),
                excerpt=match.group("excerpt"),
                url=match.group("url"),
                author=match.group("author") or None,
            )
            if built is not None:
                items.append(built)
        return items



from functools import partial
from pathlib import Path
from typing import TypeAlias

from .providers import (
    BaiduSerp,
    BilibiliSerp,
    BingSerp,
    BraveSerp,
    DouyinSerp,
    DuckDuckGoSerp,
    EngineConfig,
    GoogleSerp,
    RedditSerp,
    SerpItem,
    SoSerp,
    SogouSerp,
    ThreadsSerp,
    TiktokSerp,
    ToutiaoSerp,
    WeiboSerp,
    YandexSerp,
    YouTubeSerp,
    ZhihuSerp,
)
from .providers._base import load_engine_configs

ProviderName: TypeAlias = str

PROVIDERS = {
    "baidu": BaiduSerp,
    "bilibili": BilibiliSerp,
    "bing": BingSerp,
    "brave": BraveSerp,
    "douyin": DouyinSerp,
    "duckduckgo": DuckDuckGoSerp,
    "google": GoogleSerp,
    "reddit": RedditSerp,
    "so": SoSerp,
    "sogou": SogouSerp,
    "threads": ThreadsSerp,
    "tiktok": TiktokSerp,
    "toutiao": ToutiaoSerp,
    "weibo": WeiboSerp,
    "yandex": YandexSerp,
    "youtube": YouTubeSerp,
    "zhihu": ZhihuSerp,
}

SUPPORTED_ENGINES: tuple[EngineConfig, ...] = load_engine_configs()


def get_provider(name: ProviderName):
    return PROVIDERS[name]()


def parse_har(provider: str, har_path: str | Path) -> list[SerpItem]:
    return get_provider(provider).parse_har_file(har_path)


def search(keyword: str, provider: str = "google") -> list[SerpItem]:
    return get_provider(provider).query(keyword)


baidu = partial(search, provider="baidu")
bing = partial(search, provider="bing")
google = partial(search, provider="google")
